import rpyc
from pdb import pm
from argparse import ArgumentParser

from miasm2.analysis.machine import Machine
from miasm2.analysis.binary import Container
from miasm2.analysis.depgraph import DependencyGraph, DependencyNode
from miasm2.expression.expression import *
from miasm2.core.utils import *


conn = rpyc.classic.connect('localhost', 4455)

# Parse arguments
parser = ArgumentParser("Disassemble a binary")
parser.add_argument('filename', help="File to disassemble")
args = parser.parse_args()

cont = Container.from_stream(open(args.filename))
bs = cont.bin_stream

machine = Machine(cont.arch)
mn, dis_engine = machine.mn, machine.dis_engine
ira, ir = machine.ira, machine.ir
mdis = dis_engine(bs)

ira_cache = {}

def find_call(ir_arch, addr, func_addr):
    """
    Returns irb, index which call @func_addr
    """

    irbs = ir_arch.getby_offset(addr)
    out = set()
    for irb in irbs:
        if len(irb.irs) < 2:
            continue
        assignblk = irb.irs[-2]
        for dst, src in assignblk.iteritems():
            if not isinstance(src, ExprOp):
                    continue
            if not src.op.startswith('call_func'):
                continue
            out.add((irb, len(irb.irs)-2))
    assert(len(out) == 1)
    irb, index = list(out)[0]
    return irb, index

def get_arg_depgraph(addr, func_addr):
    if func_addr in ira_cache:
        ir_arch = ira_cache[func_addr]
    else:
        blocs = mdis.dis_multibloc(func_addr)
        ir_arch = ira(mdis.symbol_pool)
        for b in blocs:
            ir_arch.add_bloc(b)
        ira_cache[func_addr] = ir_arch

    g_dep = DependencyGraph(ir_arch, follow_call=False)
    lbl_head = ir_arch.getby_offset(func_addr)

    irb, index = find_call(ir_arch, addr, func_addr)

    element1 = ir_arch.arch.regs.RDX
    element2 = ir_arch.arch.regs.R8

    elements = [element1, element2]
    g_list = g_dep.get(irb.label, elements, index, lbl_head)
    for dep in g_list:
        emul_result = dep.emul()
        values = []
        for element in elements:
            value = emul_result[element]
            values.append(value)
        yield values

all_addr_dec_str = [0x16918]
off_imgbase = 0x10000
for i, (addr_dec_str) in enumerate(all_addr_dec_str):
    for ref in conn.modules.idautils.XrefsTo(addr_dec_str):
        func = conn.modules.idaapi.get_func(ref.frm)
        if func is None:
            print 'Cannot find function'
            continue
        solutions = get_arg_depgraph(ref.frm - off_imgbase, func.startEA - off_imgbase)
        for solution in solutions:
            addr, l = solution
            print "Solution for %r: %-10s %-10s" % (hex(ref.frm), addr, l)
